// @flow
import fs from 'fs-extra';
import path from 'path';
import { AbstractVinyl } from '../consumer/component/sources';
import { AUTO_GENERATED_STAMP, AUTO_GENERATED_MSG } from '../constants';
import type { PathOsBased } from '../utils/path';
import { BitId } from '../bit-id';

export default class LinkFile extends AbstractVinyl {
  override: ?boolean = false;
  writeAutoGeneratedMessage: ?boolean = true;
  srcPath: ?string; // existing path where the link is pointing to (needed for logging purposes)
  componentId: ?BitId; // needed for logging purposes

  async write(): Promise<string> {
    if (!this.override && fs.existsSync(this.path)) {
      const fileContent = fs.readFileSync(this.path).toString();
      if (!fileContent.includes(AUTO_GENERATED_STAMP)) return Promise.resolve(this.path);
    }

    const data = this.writeAutoGeneratedMessage ? AUTO_GENERATED_MSG + this.contents : this.contents;
    await fs.outputFile(this.path, data);
    return Promise.resolve(this.path);
  }

  static load({
    filePath,
    base,
    content,
    override = false,
    writeAutoGeneratedMessage = true,
    srcPath,
    componentId
  }: {
    filePath: PathOsBased,
    base?: string,
    content: string,
    override?: boolean,
    writeAutoGeneratedMessage?: boolean,
    srcPath?: string,
    componentId?: BitId
  }): LinkFile {
    const linkFile = new LinkFile({
      base: base || path.dirname(filePath),
      path: filePath,
      contents: Buffer.from(content)
    });
    linkFile.override = override;
    linkFile.writeAutoGeneratedMessage = writeAutoGeneratedMessage;
    linkFile.srcPath = srcPath;
    linkFile.componentId = componentId;
    return linkFile;
  }
}
